﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace ClientServerStateManagementWebApp
{
    /// <summary>
    /// Handles server side of the Default page.
    /// </summary>
    public partial class Default : System.Web.UI.Page
    {
        // constants
        protected const String VIEW_STATE_VALUE = "ViewStateValue";
        protected const String COOKIE_LAST_VISITED_DATE = "CookieLastVisitedDate";
        protected const String QUERY_STRING_VALUE_KEY = "keyQS";
        protected const String APPLICATION_STATE_KEY = "AppStateKey";
        protected const String SESSION_STATE_KEY = "SessionStateKey";

        protected void Page_Load(object sender, EventArgs e)
        {
            // display cookies functionality
            HandleCookies();
            // display query string value, in case there is any
            HandleQueryString();

            if (!Page.IsPostBack)
            {
                // bind these labels so their values are bound in the aspx is displayed
                lblApplicationState.DataBind();
                lblSessionState.DataBind();
            }
        }

        /// <summary>
        /// Get value from the query string with key QUERY_STRING_VALUE_KEY.
        /// </summary>
        private void HandleQueryString()
        {
            // use HtmlEnconde just to make sure no malicious string is set here
            String queryStringValue = Server.HtmlEncode(Request.QueryString[QUERY_STRING_VALUE_KEY]);
            if (!String.IsNullOrWhiteSpace(queryStringValue))
            {
                lblbQueryStringValue.Text = "Query String value is: " + queryStringValue;
            }
            else
            {
                lblbQueryStringValue.Text = "There is no query string value.";
            }
        }

        /// <summary>
        /// Show how cookies can be used. Note that they will not work in localhost because they require
        /// two names and a dot between them. Look at the following link at Stackoverflow: http://stackoverflow.com/questions/390219/why-wont-asp-net-create-cookies-in-localhost
        /// </summary>
        private void HandleCookies()
        {
            // if there is a cookie, show its value which is the last time the site was visited
            if (Request.Cookies[COOKIE_LAST_VISITED_DATE] != null)
            {
                // use HtmlEnconde just to make sure no malicious string is set here
                lblLastVisitedDate.Text = Server.HtmlEncode(Request.Cookies[COOKIE_LAST_VISITED_DATE].Value);
            }
            // if there is no cookie or it has expired, show an appropriate message
            else
            {
                lblLastVisitedDate.Text = "There was no last visit, meaning no cookie was set for this value, or the cookie has expired.";
            }

            // set the cookie to be displayed above holding the current time, meaning the last time this site was visited
            Response.Cookies[COOKIE_LAST_VISITED_DATE].Value = new DateTime().ToLongDateString();
            Response.Cookies[COOKIE_LAST_VISITED_DATE].Expires = new DateTime().AddDays(1);
        }

        /// <summary>
        /// Add a string coming from a text field to the session state and show it
        /// in a label.
        /// </summary>
        /// <param name="sender">Object which dispatched this event.</param>
        /// <param name="e">Event Arguments.</param>
        protected void btnAddToSessionState_Click(object sender, EventArgs e)
        {
            Session[SESSION_STATE_KEY] = txtSessionState.Text;
            // use HtmlEnconde just to make sure no malicious string is set here
            lblSessionState.Text = Server.HtmlEncode((String)Session[SESSION_STATE_KEY]);
            lblSessionState.DataBind();
        }

        /// <summary>
        /// Add a string coming from a text field to the application state and show it
        /// in a label.
        /// </summary>
        /// <param name="sender">Object which dispatched this event.</param>
        /// <param name="e">Event Arguments.</param>
        protected void btnAddToApplicationState_Click(object sender, EventArgs e)
        {
            Application[APPLICATION_STATE_KEY] = txtApplicationState.Text;
            // use HtmlEnconde just to make sure no malicious string is set here
            lblApplicationState.Text = Server.HtmlEncode((String)Application[APPLICATION_STATE_KEY]);
            lblApplicationState.DataBind();
        }


        /// <summary>
        /// Save a view state value and put it in a label to be displayed.
        /// </summary>
        /// <param name="sender">Object which dispatched this event.</param>
        /// <param name="e">Event Arguments.</param>
        protected void btnProcessViewStateValue_Click(object sender, EventArgs e)
        {
            ViewStateHolder holder = new ViewStateHolder();
            holder.Value = txtViewState.Text;
            ViewState[VIEW_STATE_VALUE] = holder;
            lblViewStateValue.DataBind();
        }

        /// <summary>
        /// Store a hidden field value and display it in a label.
        /// </summary>
        /// <param name="sender">Object which dispatched this event.</param>
        /// <param name="e">Event Arguments.</param>
        protected void btnProcessHiddenFieldValue_Click(object sender, EventArgs e)
        {
            hdnHiddenField.Value = txtHiddenField.Text;
            lblHiddenField.Text = hdnHiddenField.Value;
        }

        /// <summary>
        /// Redirect this very page adding a query string value.
        /// </summary>
        /// <param name="sender">Object which dispatched this event.</param>
        /// <param name="e">Event Arguments.</param
        protected void btnAddQueryString_Click(object sender, EventArgs e)
        {
            // redirect to the same page only to pass a query string value
            Response.Redirect(Request.Url.GetLeftPart(UriPartial.Path) + "?" + QUERY_STRING_VALUE_KEY + "=32");
        }
    }
}

/// <summary>
/// // Serializable class which can be persisted in the View State
/// </summary>
[Serializable]
public class ViewStateHolder
{
    public String Value { get; set; }
}